#include <stdlib.h>
#include <SDL/SDL.h>
#include <SDL/SDL_mixer.h>

void handleKey(SDL_KeyboardEvent key);


Mix_Music *music;
Mix_Chunk *sample1, *sample2, *sample3;

int main(int argc, char **argv)
{
  SDL_Surface *screen;
  SDL_Event event;
  Uint8 *keys;
  int quit = 0;

  int frequency = 44100;
  Uint16 format = MIX_DEFAULT_FORMAT; /* Corrisponde a AUDIO_S16SYS */
  int channels = MIX_DEFAULT_CHANNELS; /* Stereo */
  int buffers = 2048;

  Uint8 left = 127;
  int volume = MIX_MAX_VOLUME;

  char *music_file, *sample1_file, *sample2_file, *sample3_file;

  /* Operiamo il parsing dei parametri dati da shell */
  if (argc == 1) {
    music_file = "music.ogg";
    sample1_file = "sample1.ogg";
    sample2_file = "sample2.ogg";
    sample3_file = "sample3.ogg";
  }
  else if (argc == 5) {
    music_file = argv[1];
    sample1_file = argv[2];
    sample2_file = argv[3];
    sample3_file = argv[4];
  }
  else {
    printf("La sintassi corretta : %s musica campione1 campione2 campione3\n", argv[0]);
    exit(-1);
  }

  /* Inizializzazione del sottosistema audio e video */
  if (SDL_Init(SDL_INIT_VIDEO|SDL_INIT_AUDIO) < 0) {
    printf("Errore init SDL: %s\n", SDL_GetError());
    exit (-1);
  }

  /* All'uscita del programma viene eseguito SDL_Quit per risistemare le cose */
        atexit(SDL_Quit);

  /* Apriamo una finestra per catturare gli eventi dell'utente */
  if ((screen = SDL_SetVideoMode(320, 240, 0, 0)) == NULL) {
    printf("Errore apertura video: %s\n", SDL_GetError());
    exit (-1);
  }

  /* Apriamo il dispositivo audio */
  if (Mix_OpenAudio(frequency, format, channels, buffers) == -1) {
    printf("Errore apertura dispositivo audio: %s\n", Mix_GetError());
    exit(-1);
  }

  /* Ridimensioniamo il numero di canali allocati, ce ne bastano solo 4 */
  Mix_AllocateChannels(4);

  /* Analizziamo cosa ci ha effettivamente dato a disposizione il sistema */
  int numtimesopened;
  numtimesopened = Mix_QuerySpec(&frequency, &format, &channels);
  if (!numtimesopened)
    printf("Errore con la Mix_QuerySpec(): %s\n", Mix_GetError());
  else {
    char *format_str="Sconosciuto";
    switch (format) {
      case AUDIO_U8:
        format_str="U8";
        break;
      case AUDIO_S8:
        format_str="S8";
        break;
      case AUDIO_U16LSB:
        format_str="U16LSB";
        break;
      case AUDIO_S16LSB:
        format_str="S16LSB";
        break;
      case AUDIO_U16MSB:
        format_str="U16MSB";
        break;
      case AUDIO_S16MSB:
        format_str="S16MSB";
        break;
    }

  printf("Il dispositivo  stato aperto %d volte\n", numtimesopened);
  printf("Specifiche audio ottenute dal sistema:\n");
  printf("Frequenza = %d\n", frequency);
  printf("Formato = %s\n", format_str);
  printf("Canali = %d\n\n", channels);
  }

  /* Carichiamo la musica */
  if ((music = Mix_LoadMUS(music_file)) == NULL)
    printf("Errore caricamento musica : %s\n", Mix_GetError());

  /* Carichiamo i campioni audio */
  if ((sample1 = Mix_LoadWAV(sample1_file)) == NULL)
    printf("Errore caricamento campione 1: %s\n", Mix_GetError());
  if ((sample2 = Mix_LoadWAV(sample2_file)) == NULL)
    printf("Errore caricamento campione 2: %s\n", Mix_GetError());
  if ((sample3 = Mix_LoadWAV(sample3_file)) == NULL)
    printf("Errore caricamento campione 3: %s\n", Mix_GetError());

  /* Registriamo un effetto di panning centrale */
  Mix_SetPanning(MIX_CHANNEL_POST, 127, 127);

  /* Ciclo di gestione degli eventi dell'utente */
  while (!quit) {
    while (SDL_PollEvent(&event)) {
      switch (event.type) {
      case SDL_QUIT:
        quit = 1;
        break;
      case SDL_KEYDOWN:
        handleKey(event.key);
        break;
      }
      if (event.key.keysym.sym == SDLK_ESCAPE || event.key.keysym.sym == SDLK_q)
        quit = 1;
    }

    keys = SDL_GetKeyState(NULL);

    if (keys[SDLK_UP]) {
      if (volume < MIX_MAX_VOLUME) {
        volume += 4;
        Mix_Volume(-1, volume);
        Mix_VolumeMusic(volume);
        printf("Volume = %d\n", volume);
      }
    }

    if (keys[SDLK_DOWN]) {
      if (volume > 0) {
        volume -= 4;
        Mix_Volume(-1, volume);
        Mix_VolumeMusic(volume);
        printf("Volume = %d\n", volume);
      }
    }

    if (keys[SDLK_LEFT]) {
      if (left < 250) {
        left += 4;
        Mix_SetPanning(MIX_CHANNEL_POST, left, 254-left);
        printf("Panning = L:%d, R:%d\n", left, 254-left);
      }
    }

    if (keys[SDLK_RIGHT]) {
      if (left > 4) {
        left -= 4;
        Mix_SetPanning(MIX_CHANNEL_POST, left, 254-left);
        printf("Panning = L:%d, R:%d\n", left, 254-left);
      }
    }

    /* Facciamo respirare la CPU tra un poll ed un altro */
    SDL_Delay(50);
  }

  /* Liberiamo la memoria allocata per i campioni e la musica e chiudiamo il dispositivo audio */
  Mix_FreeChunk(sample3);
  Mix_FreeChunk(sample2);
  Mix_FreeChunk(sample1);
  Mix_FreeMusic(music);
  Mix_CloseAudio();
  SDL_Quit();

  return 0;
}

/* Gestisce la struttura SDL_KeyboardEvent per rispondere all'input da tastiera */
void handleKey(SDL_KeyboardEvent key)
{
  switch(key.keysym.sym) {
  case SDLK_c:
    Mix_SetPanning(MIX_CHANNEL_POST, 127, 127);
    printf("Panning = Centrale\n");
    break;
  case SDLK_f:
    if (Mix_PlayingMusic())
      Mix_FadeOutMusic(3000);
    else
      Mix_FadeInMusic(music, -1, 3000);
    break;
  case SDLK_m:
    if (Mix_PlayingMusic())
      Mix_HaltMusic();
    else
      Mix_PlayMusic(music, -1);
    break;
  case SDLK_p:
    if (Mix_PausedMusic())
      Mix_ResumeMusic();
    else
      Mix_PauseMusic();
    break;
  case SDLK_1:
  case SDLK_KP1:
    Mix_PlayChannel(1, sample1, 0);
    break;
  case SDLK_2:
  case SDLK_KP2:
    Mix_PlayChannel(2, sample2, 0);
    break;
  case SDLK_3:
  case SDLK_KP3:
    Mix_PlayChannel(3, sample3, 0);
    break;
  }
}
